@@ -7,7 +7,7 @@
               (C) 1999 Lars Knoll (knoll@kde.org)
               (C) 1999 Antti Koivisto (koivisto@kde.org)
               (C) 2001 Dirk Mueller (mueller@kde.org)
-    Copyright (C) 2003 Apple Computer, Inc.
+    Copyright (C) 2004 Apple Computer, Inc.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -466,7 +466,8 @@ void HTMLTokenizer::scriptHandler()
     // We are inside a <script>
     bool doScriptExec = false;
     CachedScript* cs = 0;
-    if (!scriptSrc.isEmpty()) {
+    // don't load external scripts for standalone documents (for now)
+    if (!scriptSrc.isEmpty() && parser->doc()->part()) {
         // forget what we just got; load from src url instead
         if ( !parser->skipMode() ) {
             if ( (cs = parser->doc()->docLoader()->requestScript(scriptSrc, scriptSrcCharset) ))
@@ -550,7 +551,7 @@ void HTMLTokenizer::scriptExecution( const QString& str, QString scriptURL,
                                      int baseLine)
 {
 #if APPLE_CHANGES
-    if (!view->part())
+    if (!view || !view->part())
         return;
 #endif
     bool oldscript = script;
@@ -731,20 +732,19 @@ void HTMLTokenizer::parseEntity(DOMStringIt &src, QChar *&dest, bool start)
 
         case Hexadecimal:
         {
-            int ll = kMin(src.length(), 9-cBufferPos);
+            int ll = kMin(src.length(), 8);
             while(ll--) {
                 QChar csrc(src->lower());
                 cc = csrc.cell();
 
                 if(csrc.row() || !((cc >= '0' && cc <= '9') || (cc >= 'a' && cc <= 'f'))) {
-                    Entity = SearchSemicolon;
                     break;
                 }
                 EntityUnicodeValue = EntityUnicodeValue*16 + (cc - ( cc < 'a' ? '0' : 'a' - 10));
                 cBuffer[cBufferPos++] = cc;
                 ++src;
             }
-            if(cBufferPos == 9) Entity = SearchSemicolon;
+            Entity = SearchSemicolon;
             break;
         }
         case Decimal:
@@ -911,11 +911,10 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
                     finish = true;
                     break;
                 }
-                // this is a nasty performance trick. will work for the A-Z
-                // characters, but not for others. if it contains one,
-                // we fail anyway
-                char cc = curchar;
-                cBuffer[cBufferPos++] = cc | 0x20;
+                // Use tolower() instead of | 0x20 to lowercase the char because there is no 
+                // performance gain in using | 0x20 since tolower() is optimized and 
+                // | 0x20 turns characters such as '_' into junk.
+                cBuffer[cBufferPos++] = tolower(curchar);
                 ++src;
             }
 
@@ -936,11 +935,14 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
                 else
                     // Start Tag
                     beginTag = true;
-                    
-                // Accept empty xml tags like <br/>
+
+                // Accept empty xml tags like <br/>.  We trim off the "/" so that when we call
+                // getTagID, we'll look up "br" as the tag name and not "br/".
                 if(len > 1 && ptr[len-1] == '/' )
                     ptr[--len] = '\0';
 
+                // Look up the tagID for the specified tag name (now that we've shaved off any
+                // invalid / that might have followed the name).
                 uint tagID = khtml::getTagID(ptr, len);
                 if (!tagID) {
 #ifdef TOKEN_DEBUG
@@ -973,7 +975,7 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
             while(src.length()) {
                 curchar = *src;
                 if(curchar > ' ') {
-                    if(curchar == '>')
+                    if (curchar == '<' || curchar == '>')
                         tag = SearchEnd;
                     else if(atespace && (curchar == '\'' || curchar == '"'))
                     {
@@ -1013,6 +1015,15 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
                         else {
                             attrName = QString::fromLatin1(QCString(cBuffer, cBufferPos+1).data());
                             attrNamePresent = !attrName.isEmpty();
+
+                            // This is a deliberate quirk to match Mozilla and Opera.  We have to do this
+                            // since sites that use the "standards-compliant" path sometimes send
+                            // <script src="foo.js"/>.  Both Moz and Opera will honor this, despite it
+                            // being bogus HTML.  They do not honor the "/" for other tags.  This behavior
+                            // also deviates from WinIE, but in this case we'll just copy Moz and Opera.
+                            if (currToken.id == ID_SCRIPT && curchar == '>' &&
+                                attrName == "/")
+                                currToken.flat = true;
                         }
                         
                         dest = buffer;
@@ -1028,7 +1039,10 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
                         break;
                     }
                 }
-                cBuffer[cBufferPos++] = (char) curchar | 0x20;
+                // Use tolower() instead of | 0x20 to lowercase the char because there is no 
+                // performance gain in using | 0x20 since tolower() is optimized and 
+                // | 0x20 turns characters such as '_' into junk.
+                cBuffer[cBufferPos++] = tolower(curchar);
                 ++src;
             }
             if ( cBufferPos == CBUFLEN ) {
@@ -1201,7 +1215,7 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
                 qDebug("SearchEnd");
 #endif
             while(src.length()) {
-                if(*src == '>')
+                if (*src == '>' || *src == '<')
                     break;
 
                 if (*src == '/')
@@ -1209,12 +1223,14 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
 
                 ++src;
             }
-            if(!src.length() && *src != '>') break;
+            if (!src.length() && *src != '>' && *src != '<') break;
 
             searchCount = 0; // Stop looking for '<!--' sequence
             tag = NoTag;
             tquote = NoQuote;
-            ++src;
+
+            if (*src != '<')
+                ++src;
 
             if ( !currToken.id ) //stop if tag is unknown
                 return;
@@ -1225,14 +1241,15 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
 #endif
             bool beginTag = !currToken.flat && (tagID < ID_CLOSE_TAG);
 
-            if(tagID >= ID_CLOSE_TAG)
+            if (tagID >= ID_CLOSE_TAG)
                 tagID -= ID_CLOSE_TAG;
-            else if ( beginTag && tagID == ID_SCRIPT ) {
+            else if (tagID == ID_SCRIPT) {
                 AttributeImpl* a = 0;
                 scriptSrc = QString::null;
                 scriptSrcCharset = QString::null;
                 if ( currToken.attrs && /* potentially have a ATTR_SRC ? */
-                     parser->doc()->view()->part()->jScriptEnabled() && /* jscript allowed at all? */
+		     parser->doc()->part() &&
+                     parser->doc()->part()->jScriptEnabled() && /* jscript allowed at all? */
                      view /* are we a regular tokenizer or just for innerHTML ? */
                     ) {
                     if ( ( a = currToken.attrs->getAttributeItem( ATTR_SRC ) ) )
@@ -1240,7 +1257,7 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
                     if ( ( a = currToken.attrs->getAttributeItem( ATTR_CHARSET ) ) )
                         scriptSrcCharset = a->value().string().stripWhiteSpace();
                     if ( scriptSrcCharset.isEmpty() )
-                        scriptSrcCharset = parser->doc()->view()->part()->encoding();
+                        scriptSrcCharset = parser->doc()->part()->encoding();
                     if (!(a = currToken.attrs->getAttributeItem( ATTR_LANGUAGE )))
                         a = currToken.attrs->getAttributeItem(ATTR_TYPE);
                 }
@@ -1277,6 +1294,8 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
                     script = true;
                     parseSpecial(src);
                 }
+                else if (tagID < ID_CLOSE_TAG) // Handle <script src="foo"/>
+                    scriptHandler();
                 break;
             case ID_STYLE:
                 if (beginTag) {
@@ -1415,8 +1434,7 @@ void HTMLTokenizer::write( const QString &str, bool appendData )
 //     if (Entity)
 //         parseEntity(src, dest);
 
-    while ( src.length() )
-    {
+    while (src.length() && (!parser->doc()->part() || !parser->doc()->part()->isScheduledLocationChangePending())) {
         // do we need to enlarge the buffer?
         checkBuffer();
 
@@ -1790,8 +1808,6 @@ void HTMLTokenizer::notifyFinished(CachedObject */*finishedObj*/)
         kdDebug( 6036 ) << "Finished loading an external script" << endl;
 #endif
         CachedScript* cs = cachedScript.dequeue();
-        finished = cachedScript.isEmpty();
-        if (finished) loadingExtScript = false;
         DOMString scriptSource = cs->script();
 #ifdef TOKEN_DEBUG
         kdDebug( 6036 ) << "External script is:" << endl << scriptSource.string() << endl;
@@ -1804,6 +1820,10 @@ void HTMLTokenizer::notifyFinished(CachedObject */*finishedObj*/)
         cs->deref(this);
 
 	scriptExecution( scriptSource.string(), cachedScriptUrl );
+        // cachedScript.isEmpty() can change inside the scriptExecution() call above,
+        // so don't test it until afterwards.
+        finished = cachedScript.isEmpty();
+        if (finished) loadingExtScript = false;
 
         // 'script' is true when we are called synchronously from
         // parseScript(). In that case parseScript() will take care
@@ -1818,6 +1838,11 @@ void HTMLTokenizer::notifyFinished(CachedObject */*finishedObj*/)
     }
 }
 
+bool HTMLTokenizer::isWaitingForScripts()
+{
+    return loadingExtScript;
+}
+
 void HTMLTokenizer::setSrc(const QString &source)
 {
     lineno += src.lineCount();
