@@ -32,6 +32,9 @@
 class QPainter;
 class QRect;
 
+// Uncomment to enable INCREMENTAL_REPAINTING
+#define INCREMENTAL_REPAINTING
+
 namespace DOM {
     class HTMLDocumentImpl;
     class DocumentImpl;
@@ -48,9 +51,11 @@ namespace DOM {
 
 namespace khtml {
     class RenderObject;
+    class RenderBox;
     class RenderCanvas;
     class RenderStyle;
     class RenderLineEdit;
+    class RenderPart;
     class RenderPartObject;
     class RenderWidget;
     class CSSStyleSelector;
@@ -78,7 +83,9 @@ class KHTMLView : public QScrollView
     friend class KHTMLPart;
     friend class khtml::RenderCanvas;
     friend class khtml::RenderObject;
+    friend class khtml::RenderBox;
     friend class khtml::RenderLineEdit;
+    friend class khtml::RenderPart;
     friend class khtml::RenderPartObject;
     friend class khtml::RenderWidget;
     friend class khtml::CSSStyleSelector;
@@ -136,6 +143,9 @@ public:
      */
     virtual void setHScrollBarMode ( ScrollBarMode mode );
 
+    // Sets both horizontal and vertical modes.
+    virtual void setScrollBarsMode(ScrollBarMode mode);
+    
     /**
      * Prints the HTML document.
      */
@@ -146,6 +156,17 @@ public:
      */
     void layout();
 
+    bool inLayout() const;
+    int layoutCount() const;
+
+#ifdef INCREMENTAL_REPAINTING
+    bool needsFullRepaint() const;
+#endif
+    
+#if APPLE_CHANGES
+    void resetScrollBars();
+#endif
+
 signals:
     void cleared();
 
@@ -185,9 +206,13 @@ public:
 
     void timerEvent ( QTimerEvent * );
 
+    void repaintRectangle(const QRect& r, bool immediate);
+    
 #if APPLE_CHANGES
     QWidget *topLevelWidget() const;
     QPoint mapToGlobal(const QPoint &) const;
+    void adjustViewSize();
+    void initScrollBars();
 #endif
 
     void ref() { ++_refCount; }
@@ -201,12 +226,13 @@ private:
 
     void resetCursor();
 
+#ifdef INCREMENTAL_REPAINTING
+    void scheduleRelayout();
+#else
     void scheduleRelayout(khtml::RenderObject* clippedObj=0);
+#endif
     void unscheduleRelayout();
 
-    void scheduleRepaint(int x, int y, int w, int h);
-    void unscheduleRepaint();
-    
     /**
      * Paints the HTML document to a QPainter.
      * The document will be scaled to match the width of
@@ -253,9 +279,13 @@ private:
 
     void complete();
 
+    void applyBodyScrollQuirk(khtml::RenderObject* o, ScrollBarMode& hMode, ScrollBarMode& vMode);
+    
+#ifndef INCREMENTAL_REPAINTING
     // Returns the clipped object we will repaint when we perform our scheduled layout.
     khtml::RenderObject* layoutObject() { return m_layoutObject; }
-
+#endif
+    
     // ------------------------------------- member variables ------------------------------------
  private:
     unsigned _refCount;
@@ -270,10 +300,12 @@ private:
     KHTMLViewPrivate *d;
 
     QString m_medium;   // media type
-    
+
+#ifndef INCREMENTAL_REPAINTING
     // An overflow: hidden clipped object.  If this is set, a scheduled layout will only repaint
     // the object's clipped area, and it will not do a full repaint.
     khtml::RenderObject* m_layoutObject;
+#endif
 };
 
 #endif
