@@ -1042,6 +1042,16 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
     return nil;
 }
 
+- (WebCoreBridge *)opener
+{
+    KHTMLPart *openerPart = _part->opener();
+
+    if (openerPart)
+        return KWQ(openerPart)->bridge();
+
+    return nil;
+}
+
 + (NSString *)stringWithData:(NSData *)data textEncoding:(CFStringEncoding)textEncoding
 {
     if (textEncoding == kCFStringEncodingInvalidId || textEncoding == kCFStringEncodingISOLatin1) {
